/**************************************************************************
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*
*     (c) Copyright 2008 Xilinx, Inc.
*     All rights reserved.
*
**************************************************************************/

/**************************************************************************
* Filename:     TestApp_DCache.c
*
* Description:
* This application will write to 4 different addresses, each aligned to a
* cache line.  This operation will be performed twice, once with caching
* inhibited and again with caching enabled. This will establish a known 
* pattern which can be used to investicate cache behaviour with XMD.
*
*
* MODIFICATION HISTORY:
*
* Ver    Who  Date     Changes
* ------ ---- -------- -------------------------------------------------------
* 1.00.a bh   3/30/08 Initial release
*
*
*
******************************************************************************/

#include "xparameters.h"

#include "stdio.h"

#include "xutil.h"
#include "xuartns550_l.h"

#include "xcache_l.h"
#include "xtlb_l.h"

#include "xpseudo_asm_gcc.h"

#define TS_INSTRUCTION 0
#define TS_DATA        1
#define UNCACHED_ALIAS_ADDRESS 0xB0000000

volatile unsigned *foo1, *foo2, *foo3, *foo4;

/*
 * print_foo_data:
 * Display 32 bytes of data from each of the 4 arrays.
 */
void
print_foo_data (void)
{
    int i;

    xil_printf("Wrote 32 bytes to 0x%08x \n\r", foo1);
    for (i=0; i<8; i++) {
        xil_printf("%08x", foo1[i]);
    }
    xil_printf("\n\rWrote 32 bytes to 0x%08x \n\r", foo2);
    for (i=0; i<8; i++) {
        xil_printf("%08x", foo2[i]);
    }
    xil_printf("\n\rWrote 32 bytes to 0x%08x \n\r", foo3);
    for (i=0; i<8; i++) {
        xil_printf("%08x", foo3[i]);
    }
    xil_printf("\n\rWrote 32 bytes to 0x%08x \n\r", foo4);
    for (i=0; i<8; i++) {
        xil_printf("%08x", foo4[i]);
    }
    xil_printf("\n\r");
}

/*
 * main:
 */
int 
main ()
{
    int i;
    short page;

    /* Initialize RS232_Uart - Set baudrate and number of stop bits */
    XUartNs550_SetBaud(XPAR_RS232_UART_1_BASEADDR,
                       XPAR_XUARTNS550_CLOCK_HZ, 9600);
    XUartNs550_mSetLineControlReg(XPAR_RS232_UART_1_BASEADDR,
                                  XUN_LCR_8_DATA_BITS);

    xil_printf("\n\r\n\rApplication to assist with Data Cache study:\n\r");

    /*
     * Find the TLB entry mapping the virtual address 0xB0000000 -> 0xB0000000
     * This mapping was created when the standalone BSP stated.
     */
    page = XTlb_FindPage(UNCACHED_ALIAS_ADDRESS, 0, TS_DATA);
    /*
     * Change the mapping.
     * Map virtual address 0xB0000000 -> physical 0x00000000
     * Creating an alias for physical address 0x00000000 (it is also
     * represented by virtual address 0x00000000)
     */
    XTlb_SetAddrTxlat(page, 0x00000000, 0, XTLB_256MB_PAGE);
    xil_printf("\n\rTLB Alias 0xB0000000 -> 0x00000000 (uncached) "
               "created\n\r\n\r");

    foo1 = (unsigned*)0x000043E0;
    foo2 = (unsigned*)0x000143E0;
    foo3 = (unsigned*)0x000242E0;
    foo4 = (unsigned*)0x000343E0;

    xil_printf("Data cache is presently disabled for 0x00000000\n\r\n\r");

    /*
     * Fill memory with this known pattern.
     * These addresses are presently cache-inhibited
     */
    for (i=0; i<8; i++) {
        foo1[i] = 0xDEADBEEF;
        foo2[i] = 0xDEADBEEF;
        foo3[i] = 0xDEADBEEF;
        foo4[i] = 0xDEADBEEF;
    }
    print_foo_data(); /* Display written contents */

    /*
     * Enable data cache for the first 256MB of memory, 
     * virtual address 0x00000000
     */
    XCache_EnableDCache(0x80000000);
    xil_printf("\n\rData cache enabled for 0x00000000 (256MB)\n\r\n\r");

    /*
     * Fill the data cache with this known pattern
     * These cache lines are now "Dirty" (not written to memory). The
     * pattern previously writtten to memory is not overwritten.
     */
    for (i=0; i<8; i++) {
        foo1[i] = i;
        foo2[i] = i+8;
        foo3[i] = i+16;
        foo4[i] = i+32;
    }
    print_foo_data(); /* Display written contents */

    /* Wait here forever */
    while (1) {;}
}
