/**************************************************************************
*
*     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
*     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
*     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
*     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
*     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*     FOR A PARTICULAR PURPOSE.
*     
*     (c) Copyright 2007 Xilinx, Inc.
*     All rights reserved.
*
**************************************************************************/

/**************************************************************************
* Filename:     main.c
* 
* Description:    
* This program demonstrates the use of malloc_log_print() to debug memory
* allocation with the aide of instrumented malloc() and free() functions.
* The linker option --wrap malloc --wrap free is used so that calls to
* malloc and free in this source file result in __wrap_malloc and __wrap_free
* actually being called.  See ppc_debug_malloc.c
* 
*
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -------------------------------------------------------
*
*
*
* $Id: main.c,v 1.1 2007/11/14 21:50:24 bhill Exp $
******************************************************************************/


/***************************** Include Files *********************************/

// Located in: ppc405_0/include/xparameters.h
#include "xparameters.h"

#include "stdio.h"
#include "stdlib.h"

#include "xutil.h"
#include "xuartns550_l.h"

#include <ppc-asm.h>

extern void malloc_log_print (void);

//====================================================

/*
 * MemWaster1:
 * malloc() some memory.  A portion of the allocated memory is freed.
 */
int
MemWaster1 (void)
{
    int i;
    void * allocated;

    for (i = 0; i < 40; i++) {
        allocated = malloc(32);
        if (allocated == NULL) {
            printf("%s %s:%d malloc failed!\n\r", __FUNCTION__,
                   __FILE__, __LINE__);
            return XTRUE;
        }
        /* Keep some - don't free them all */
        if ((i % 3) == 0) {
            continue;
        }
        /* free the memory we just allocated */
        free (allocated);
    }
    return XFALSE;
}

/*
 * MemWaster2:
 * malloc() some memory.
 */
int
MemWaster2 (void)
{
    int i;
    void * allocated;

    for (i = 0; i < 45; i++) {
        allocated = malloc(58);
        if (allocated == NULL) {
            printf("%s %s:%d malloc failed!\n\r", __FUNCTION__,
                   __FILE__, __LINE__);
            return XTRUE;
        }
    }
    return XFALSE;
}

/*
 * MemWaster3:
 * malloc() some memory.
 */
int
MemWaster3 (void)
{
    void * allocated;

    allocated = malloc(10);
    if (allocated == NULL) {
        printf("%s %s:%d malloc failed!\n\r", __FUNCTION__,
               __FILE__, __LINE__);
        return XTRUE;
    }
    free (allocated);

    allocated = malloc(100);
    if (allocated == NULL) {
        printf("%s %s:%d malloc failed!\n\r", __FUNCTION__,
               __FILE__, __LINE__);
        return XTRUE;
    }

    return XFALSE;
}

/*
 * MemWaster4:
 * malloc() some memory.
 */
int
MemWaster4 (void)
{
    void * allocated;

    allocated = malloc(100);
    if (allocated == NULL) {
        printf("%s %s:%d malloc failed!\n\r", __FUNCTION__,
               __FILE__, __LINE__);
        return XTRUE;
    }
    free (allocated);

    return XFALSE;
}

/*
 * main:
 */
int
main (void)
{
   void *allocated1, *allocated2;
   int status;

   /* Initialize RS232_Uart - Set baudrate and number of stop bits */
   XUartNs550_SetBaud(XPAR_RS232_UART_1_BASEADDR,
                      XPAR_XUARTNS550_CLOCK_HZ, 9600);
   XUartNs550_mSetLineControlReg(XPAR_RS232_UART_1_BASEADDR,
                                 XUN_LCR_8_DATA_BITS);
   print("-- Entering main() --\r\n");

   status = MemWaster1();
   if (status) {
       malloc_log_print();
       return 1;
   }
   status = MemWaster2();
   if (status) {
       malloc_log_print();
       return 1;
   }
   status = MemWaster3();
   if (status) {
       malloc_log_print();
       return 1;
   }
   status = MemWaster4();
   if (status) {
       malloc_log_print();
       return 1;
   }

   print("-- Exiting main() --\r\n");
   return 0;
}

