###############################################################################
##
## Copyright (c) 2007, 2008 Xilinx, Inc.  All rights reserved.
##
## Xilinx, Inc.
## XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" AS A
## COURTESY TO YOU. BY PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
## ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION OR
## STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS IMPLEMENTATION
## IS FREE FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE RESPONSIBLE
## FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE FOR YOUR IMPLEMENTATION.
## XILINX EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH RESPECT TO
## THE ADEQUACY OF THE IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO
## ANY WARRANTIES OR REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
## FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY
## AND FITNESS FOR A PARTICULAR PURPOSE.
##
## Filename:    csum16.tcl
##
## Description:
## Calculate 16 bit ones-complement checksum
##
## $Id: csum16.tcl,v 1.1 2008/01/16 21:59:07 bhill Exp $
##
###############################################################################

puts "csum16"

# csum16
#  baseaddr  start address of data to sum in memory
#  bytes     number of bytes to sum.
#  initial   initial csum (useful for pseudo header)
#  skip      a word to optionally skip (csum field)
proc csum16 {baseaddr bytes initial skip} {
    set addr $baseaddr
    set word 0

    # start with initial checksum value
    set sum  $initial

    # how many 16-bit words ? (this is rounded up)
    set words [expr [expr $bytes + 1] / 2]

    # sum the specified number of 16-bit words in memory
    while {$word < $words} {
        # read a word from memory
        set mval [format "0x%04x" [mem_read_hwd $addr]]

        # pad with '0' if not an even number of bytes
        if {$bytes == 1} {
            set mval [expr $mval & 0xFF00]
        }

        # Add this word to the sum
        set sum [expr $sum + $mval]

        # fold in any overflow
        set sumx [expr $sum >> 16]
        while {$sumx > 0} {
            set sum [expr $sum & 0xFFFF]
            set sum [expr $sum + $sumx]
            set sumx [expr $sum >> 16]
        }

        # next word
        incr word 1
        set bytes [expr $bytes - 2]

        # skip any word specified, if specified. Useful in
        # skipping the checksum field
        if {$skip && $word == $skip} {
            # skip TCP CSUM
            incr word 1
            incr addr 2
            set bytes [expr $bytes - 2]
        }

        set addr [format "0x%08x" [expr $addr + 2]]
    }

    # compliment the sum
    set sum [expr [expr ~$sum] & 0xFFFF]
    set sum [format "0x%04x" $sum]

    return $sum
}
