###############################################################################
##
## Copyright (c) 2007 Xilinx, Inc.  All rights reserved.
##
## Xilinx, Inc.
## XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" AS A
## COURTESY TO YOU. BY PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
## ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION OR
## STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS IMPLEMENTATION
## IS FREE FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE RESPONSIBLE
## FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE FOR YOUR IMPLEMENTATION.
## XILINX EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH RESPECT TO
## THE ADEQUACY OF THE IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO
## ANY WARRANTIES OR REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
## FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY
## AND FITNESS FOR A PARTICULAR PURPOSE.
##
## Filename:	emaclite.tcl
## Author:      Brian Hill
##
## Description:
## Print XPS emaclite controller registers
##
## $Id: emaclite.tcl,v 1.2 2008/04/21 20:48:25 bhill Exp $
##
###############################################################################

puts "emaclite_print"

# decode an Ethertype to an English name
#
proc decode_ethertype {ethertype} {
    if {$ethertype <= 0x5DC} {
        puts "LENGTH: $ethertype"
    } else {
        if {$ethertype} {
            puts -nonewline "ETHERTYPE: $ethertype "
        }
        switch $ethertype {
            0x0800  {puts "IPv4" }
            0x0806  {puts "IPv4_ARP" }
            0x8035  {puts "Inverse_ARP " }
            0x86DD  {puts "IPv6 " }
            0x880B  {puts "PPP " }
            0x8847  {puts "MPLS_Unicast " }
            0x8848  {puts "MPLS_Multicast " }
            0x8863  {puts "PPPoE_Discovery " }
            0x8864  {puts "PPPoE_Session " }
            0x9000  {puts "Loopback " }
            default {puts "" }
        }
    }
}

# Print EmacLite registers -with- decode
#
proc emaclite_print {base} {
    # Read register offset definitions
    global env
    source "$env(XMD_USER_TCLDIR)/include/xemaclite_l.tcl"

    # Print first few words of Transmit Buffer
    #
    set addr [expr $base + $XEL_TXBUFF_OFFSET]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    set addr [format "0x%08x" $addr]
    puts "$addr TXBUFF W0  $mval"
    set mac_addr $mval
    incr addr 0x4

    set mval [format "0x%08x" [mem_read_word $addr] ]
    set addr [format "0x%08x" $addr]
    puts -nonewline "$addr TXBUFF W1  $mval "
    puts -nonewline "DST_MACADDR: $mac_addr"
    set mac_addr [expr [expr $mval & 0xffff0000] >> 16]
    set mac_addr [format "%04x" $mac_addr]
    puts $mac_addr
    set mac_addr [expr $mval & 0xFFFF]
    set mac_addr [format "0x%04x" $mac_addr]
    incr addr 4

    set mval [format "0x%08x" [mem_read_word $addr] ]
    set addr [format "0x%08x" $addr]
    puts -nonewline "$addr TXBUFF W2  $mval SRC_MACADDR: $mac_addr"
    set mac_addr [format "%08x" $mval]
    puts $mac_addr
    incr addr 4

    set mval [format "0x%08x" [mem_read_word $addr] ]
    set addr [format "0x%08x" $addr]
    puts -nonewline "$addr TXBUFF W3  $mval "
    set ethertype_len [format "0x%04x" [expr $mval >> 16]]
    decode_ethertype $ethertype_len
    incr addr 4

    # Tx Packet Length
    #
    set addr [expr $base + $XEL_TPLR_OFFSET]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    set addr [format "0x%08x" $addr]
    puts "$addr TPLR       $mval"
    incr addr 4

    # Global interrupt enable
    #
    set mval [format "0x%08x" [mem_read_word $addr] ]
    set addr [format "0x%08x" $addr]
    puts "$addr GIER       $mval"
    incr addr 4

    # Transmit Status
    #
    set mval [format "0x%08x" [mem_read_word $addr] ]
    set addr [format "0x%08x" $addr]
    puts -nonewline "$addr TSR        $mval "
    if {$mval & $XEL_TSR_XMIT_BUSY_MASK} {
        puts -nonewline "XMIT_BUSY "
    }
    if {$mval & $XEL_TSR_PROGRAM_MASK} {
        puts -nonewline "PROGRAM "
    }
    if {$mval & $XEL_TSR_XMIT_IE_MASK} {
        puts -nonewline "XMIT_IE "
    }
    if {$mval & $XEL_TSR_XMIT_ACTIVE_MASK} {
        puts -nonewline "XMIT_ACTIVE "
    }
    puts ""

    # Receive Buffer
    #
    puts ""
    set addr [expr $base + $XEL_RXBUFF_OFFSET]
    set addr [format "0x%08x" $addr]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    puts "$addr RXBUFF W0  $mval"
    set mac_addr $mval
    incr addr 4

    set mval [format "0x%08x" [mem_read_word $addr] ]
    set addr [format "0x%08x" $addr]
    puts -nonewline "$addr RXBUFF W1  $mval "
    puts -nonewline "DST_MACADDR: $mac_addr"
    set mac_addr [expr [expr $mval & 0xffff0000] >> 16]
    set mac_addr [format "%04x" $mac_addr]
    puts $mac_addr
    set mac_addr [expr $mval & 0xFFFF]
    set mac_addr [format "0x%04x" $mac_addr]
    incr addr 4

    set mval [format "0x%08x" [mem_read_word $addr] ]
    set addr [format "0x%08x" $addr]
    puts -nonewline "$addr RXBUFF W2  $mval SRC_MACADDR: $mac_addr"
    set mac_addr [format "%08x" $mval]
    puts $mac_addr
    incr addr 4

    set mval [format "0x%08x" [mem_read_word $addr] ]
    set addr [format "0x%08x" $addr]
    puts -nonewline "$addr RXBUFF W3  $mval "
    set ethertype_len [format "0x%04x" [expr $mval >> 16]]
    decode_ethertype $ethertype_len

    # Rx Status Register
    #
    set addr [expr $base + $XEL_RSR_OFFSET]
    set addr [format "0x%08x" $addr]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    puts -nonewline "$addr RSR        $mval "
    if {$mval & $XEL_RSR_RECV_DONE_MASK} {
        puts -nonewline "RECV_DONE "
    }
    if {$mval & $XEL_RSR_RECV_IE_MASK} {
        puts -nonewline "RECV_IE "
    }
    puts ""
}

