###############################################################################
##
## Copyright (c) 2007, 2008 Xilinx, Inc.  All rights reserved.
##
## Xilinx, Inc.
## XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" AS A
## COURTESY TO YOU. BY PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
## ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION OR
## STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS IMPLEMENTATION
## IS FREE FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE RESPONSIBLE
## FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE FOR YOUR IMPLEMENTATION.
## XILINX EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH RESPECT TO
## THE ADEQUACY OF THE IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO
## ANY WARRANTIES OR REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
## FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY
## AND FITNESS FOR A PARTICULAR PURPOSE.
##
## Filename:    lldma.tcl
## Author:      Brian Hill
##
## Description:
## Print SDMA peripheral registers, decoding interesting configuration and
## status bits.
## To use with HDMA (DCR rather than PLB registers) example:
## xmd: connect ppc hw -debugdevice dcrstartadr 0xa0000000
## lldma_mm_print 0xa0000200
##
## $Id: lldma.tcl,v 1.3 2008/04/02 19:27:50 bhill Exp $
##
###############################################################################

puts "lldma_mm_print"
puts "lldma_desc_print"

# Print RX or TX ring info
proc lldma_ring_print {base} {
    # Read register offset definitions
    global env
    source "$env(XMD_USER_TCLDIR)/include/xlldma_hw.tcl"

    set addr [format "0x%08x" [expr $base + $XLLDMA_NDESC_OFFSET]]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    puts "$addr NDESC $mval"

    set addr [format "0x%08x" [expr $base + $XLLDMA_BUFA_OFFSET]]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    puts "$addr BUFA  $mval"

    set addr [format "0x%08x" [expr $base + $XLLDMA_BUFL_OFFSET]]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    puts "$addr BUFL  $mval"

    set addr [format "0x%08x" [expr $base + $XLLDMA_CDESC_OFFSET]]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    puts "$addr CDESC $mval"

    set addr [format "0x%08x" [expr $base + $XLLDMA_TDESC_OFFSET]]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    puts "$addr TDESC $mval"

    # Display Control Register
    set addr [format "0x%08x" [expr $base + $XLLDMA_CR_OFFSET]]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    puts -nonewline "$addr CR    $mval "

    # Decode Control Register
    set cr_irq_timeout [expr [expr $mval & $XLLDMA_CR_IRQ_TIMEOUT_MASK] >> 24]
    set cr_irq_timeout [format "0x%01x" $cr_irq_timeout ]
    puts -nonewline "IRQ_TMO_$cr_irq_timeout "

    set cr_irq_count [expr [expr $mval & $XLLDMA_CR_IRQ_COUNT_MASK] >> 16]
    set cr_irq_count [format "0x%01x" $cr_irq_count ]
    puts -nonewline "IRQ_THR_$cr_irq_count "

    set cr_msb_addr [expr [expr $mval & $XLLDMA_CR_MSB_ADDR_MASK] >> 12]
    set cr_msb_addr [format "0x%01x" $cr_msb_addr ]
    puts "MSB_ADDR_$cr_msb_addr "
    puts -nonewline "                            "
    if {$mval & $XLLDMA_CR_APP_EN_MASK} {
        puts -nonewline "APP_EN "
    }
    if {$mval & $XLLDMA_CR_USE_1_BIT_CNT_MASK} {
        puts -nonewline "USE_1_BIT_CNT "
    }
    if {$mval & $XLLDMA_CR_USE_INT_ON_END_MASK} {
        puts -nonewline "USE_INT_ON_END "
    }
    if {$mval & $XLLDMA_CR_LD_IRQ_CNT_MASK} {
        puts -nonewline "LD_IRQ_CNT "
    }
    if {$mval & $XLLDMA_CR_IRQ_EN_MASK} {
        puts -nonewline "IRQ_EN "
    }
    if {$mval & $XLLDMA_CR_IRQ_ERROR_EN_MASK} {
        puts -nonewline "IRQ_ERROR_EN "
    }
    if {$mval & $XLLDMA_CR_IRQ_DELAY_EN_MASK} {
        puts -nonewline "IRQ_DELAY_EN "
    }
    if {$mval & $XLLDMA_CR_IRQ_COALESCE_EN_MASK} {
        puts -nonewline "IRQ_COALESCE_EN "
    }
    puts ""

    set addr [format "0x%08x" [expr $base + $XLLDMA_IRQ_OFFSET]]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    puts -nonewline "$addr IRQ   $mval "

    set DlyTmrValue [expr $mval & $XLLDMA_IRQ_DELAY_COUNTER_VAL_MASK]
    set DlyTmrValue [expr $DlyTmrValue >> $XLLDMA_IRQ_DELAY_COUNTER_VAL_SHIFT]
    if {$DlyTmrValue} {
        puts -nonewline "DlyTmrValue_$DlyTmrValue "
    }

    set DlyCnt [expr $mval & $XLLDMA_IRQ_DELAY_COUNTER_MASK]
    set DlyCnt [expr $DlyCnt >> $XLLDMA_IRQ_DELAY_COUNTER_SHIFT]
    if {$DlyCnt} {
        puts -nonewline "DlyCnt_$DlyCnt "
    }

    set ClscCntValue [expr $mval & $XLLDMA_IRQ_COALESCE_COUNTER_VAL_MASK]
    set ClscCntValue [expr $ClscCntValue >> $XLLDMA_IRQ_COALESCE_COUNTER_VAL_SHIFT]
    if {$ClscCntValue} {
        puts -nonewline "ClscCntValue_$ClscCntValue "
    }

    set ClscCnt [expr $mval & $XLLDMA_IRQ_COALESCE_COUNTER_MASK]
    set ClscCnt [expr $ClscCnt >> $XLLDMA_IRQ_COALESCE_COUNTER_SHIFT]
    if {$ClscCnt} {
        puts -nonewline "ClscCnt_$ClscCnt "
    }
   
    if {$mval & $XLLDMA_IRQ_WRQ_EMPTY_MASK} {
        puts -nonewline "WRQ_EMPTY "
    }
    if {$mval & $XLLDMA_IRQ_PLB_RD_ERROR_MASK} {
        puts -nonewline "PLB_RD_ERROR "
    }
    if {$mval & $XLLDMA_IRQ_PLB_WR_ERROR_MASK} {
        puts -nonewline "PLB_WR_ERROR "
    }
    if {$mval & $XLLDMA_IRQ_ERROR_MASK} {
        puts -nonewline "ERROR_IRQ "
    }
    if {$mval & $XLLDMA_IRQ_DELAY_MASK} {
        puts -nonewline "DELAY_IRQ "
    }
    if {$mval & $XLLDMA_IRQ_COALESCE_MASK} {
        puts -nonewline "COALESCE_IRQ "
    }
    puts ""

    set addr [format "0x%08x" [expr $base + $XLLDMA_SR_OFFSET]]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    puts -nonewline "$addr SR    $mval "
    if {$mval & $XLLDMA_SR_IRQ_ON_END_MASK} {
        puts -nonewline "IRQ_ON_END "
    }
    if {$mval & $XLLDMA_SR_STOP_ON_END_MASK} {
        puts -nonewline "STOP_ON_END "
    }
    if {$mval & $XLLDMA_SR_COMPLETED_MASK} {
        puts -nonewline "COMPLETED "
    }
    if {$mval & $XLLDMA_SR_SOP_MASK} {
        puts -nonewline "SOP "
    }
    if {$mval & $XLLDMA_SR_EOP_MASK} {
        puts -nonewline "EOP "
    }
    if {$mval & $XLLDMA_SR_ENGINE_BUSY_MASK} {
        puts -nonewline "ENGINE_BUSY "
    }
    puts ""
}

# Print info on a memory-mapped (not DCR) SDMA controller
proc lldma_mm_print {base} {
    # Read register offset definitions
    global env
    source "$env(XMD_USER_TCLDIR)/include/xlldma_hw.tcl"

    # Print TX Ring info
    puts "TX Ring"
    set txbase [expr $base + $XLLDMA_TX_OFFSET]
    lldma_ring_print $txbase

    # Print RX Ring info
    puts "RX Ring"
    set rxbase [expr $base + $XLLDMA_RX_OFFSET]
    lldma_ring_print $rxbase

    set addr [format "0x%08x" [expr $base + $XLLDMA_DMACR_OFFSET]]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    puts -nonewline "\n$addr DMACR $mval "
    if {$mval & $XLLDMA_DMACR_TX_PAUSE_MASK} {
        puts -nonewline "TX_PAUSE "
    }
    if {$mval & $XLLDMA_DMACR_RX_PAUSE_MASK} {
        puts -nonewline "RX_PAUSE "
    }
    if {$mval & $XLLDMA_DMACR_PLB_ERR_DIS_MASK} {
        puts -nonewline "PLB_ERR_DIS "
    }
    if {$mval & $XLLDMA_DMACR_RX_OVERFLOW_ERR_DIS_MASK} {
        puts -nonewline "RX_OVF_ERR_DIS "
    }
    if {$mval & $XLLDMA_DMACR_TX_OVERFLOW_ERR_DIS_MASK} {
        puts -nonewline "TX_OVF_ERR_DIS "
    }
    if {$mval & $XLLDMA_DMACR_TAIL_PTR_EN_MASK} {
        puts -nonewline "TAIL_PTR_EN "
    }
    if {$mval & $XLLDMA_DMACR_EN_ARB_HOLD_MASK} {
        puts -nonewline "EN_ARB_HOLD "
    }
    if {$mval & $XLLDMA_DMACR_SW_RESET_MASK} {
        puts -nonewline "SW_RESET "
    }
    puts ""
}

# Print info on an in-memory descriptor
proc lldma_desc_print {descriptor} {
    # Read register offset definitions
    global env
    source "$env(XMD_USER_TCLDIR)/include/xlldma_hw.tcl"

    puts "DMA Descriptor"

    set base [format "0x%08x" [expr $descriptor]]

    set addr [format "0x%08x" [expr $base + $XLLDMA_BD_NDESC_OFFSET]]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    puts -nonewline "\n$addr NDESC   $mval "

    set addr [format "0x%08x" [expr $base + $XLLDMA_BD_BUFA_OFFSET]]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    puts -nonewline "\n$addr BUFA    $mval "

    set addr [format "0x%08x" [expr $base + $XLLDMA_BD_BUFL_OFFSET]]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    puts -nonewline "\n$addr BUFL    $mval "

    set addr [format "0x%08x" [expr $base + $XLLDMA_BD_STSCTRL_USR0_OFFSET]]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    puts -nonewline "\n$addr STSCTRL $mval "
    if {$mval & $XLLDMA_BD_STSCTRL_ERROR_MASK} {
        puts -nonewline "ERROR "
    }
    if {$mval & $XLLDMA_BD_STSCTRL_IOE_MASK} {
        puts -nonewline "IOE "
    }
    if {$mval & $XLLDMA_BD_STSCTRL_COMPLETED_MASK} {
        puts -nonewline "COMPLETED "
    }
    if {$mval & $XLLDMA_BD_STSCTRL_SOP_MASK} {
        puts -nonewline "SOP "
    }
    if {$mval & $XLLDMA_BD_STSCTRL_EOP_MASK} {
        puts -nonewline "EOP "
    }
    if {$mval & $XLLDMA_BD_STSCTRL_BUSY_MASK} {
        puts -nonewline "BUSY "
    }
    if {$mval & 0x1} {
        puts -nonewline "CSUM_OFFLOAD "
    }
    puts ""

    set addr [format "0x%08x" [expr $base + $XLLDMA_BD_USR1_OFFSET]]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    set start_offset  [format "0x%04x" [expr $mval >> 16]]
    set insert_offset [format "0x%04x" [expr $mval & 0xFFFF]]
    puts -nonewline "$addr USR1    $mval "
    puts "TX_CSBEGIN: $start_offset TX_CSINSERT: $insert_offset"

    set addr [format "0x%08x" [expr $base + $XLLDMA_BD_USR2_OFFSET]]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    puts "$addr USR2    $mval (initial csum)"

    set addr [format "0x%08x" [expr $base + $XLLDMA_BD_USR3_OFFSET]]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    puts "$addr USR3    $mval (Rx Csum)"

    set addr [format "0x%08x" [expr $base + $XLLDMA_BD_USR4_OFFSET]]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    puts "$addr USR4    $mval (Rx Pkt Len)"

    puts ""
}
