###############################################################################
##
## Copyright (c) 2007, 2008 Xilinx, Inc.  All rights reserved.
##
## Xilinx, Inc.
## XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" AS A
## COURTESY TO YOU. BY PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
## ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION OR
## STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS IMPLEMENTATION
## IS FREE FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE RESPONSIBLE
## FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE FOR YOUR IMPLEMENTATION.
## XILINX EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH RESPECT TO
## THE ADEQUACY OF THE IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO
## ANY WARRANTIES OR REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
## FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY
## AND FITNESS FOR A PARTICULAR PURPOSE.
##
## Filename:    lltemac.tcl
## Author:      Brian Hill
##
## Description:
## Print XPS LL TEMAC registers, decoding interesting configuration and status
## bits
##
## $Id: lltemac.tcl,v 1.3 2008/04/21 20:48:25 bhill Exp $
##
###############################################################################

puts "lltemac_print"
puts "lltemac_read_phy"

# Write to an indirect register in the TEMAC
proc lltemac_writeindirect {base ireg iregval} {
    # Read register offset definitions
    global env
    source "$env(XMD_USER_TCLDIR)/include/xlltemac_hw.tcl"

    mwr [expr $base + $XTE_LSW_OFFSET] $iregval
    mwr [expr $base + $XTE_CTL_OFFSET] $ireg
}

# Read an indirect register in the TEMAC
proc lltemac_readindirect {base ireg} {
    # Read register offset definitions
    global env
    source "$env(XMD_USER_TCLDIR)/include/xlltemac_hw.tcl"

    mwr [expr $base + $XTE_CTL_OFFSET] $ireg
    return [format "0x%08x" [mem_read_word [expr $base + $XTE_LSW_OFFSET]] ]
}

# Read a PHY register thru the TEMAC via MII
proc lltemac_read_phy {base phyaddr phyreg} {
    # Read register offset definitions
    global env
    source "$env(XMD_USER_TCLDIR)/include/xlltemac_hw.tcl"

    set XTE_MIIM_PHYAD_SHIFT 5
    set XTE_RSE_MIIM_RR_MASK 0x00000002

    set MiiReg $phyreg
    set MiiReg [expr $MiiReg + [expr $phyaddr << $XTE_MIIM_PHYAD_SHIFT]]

    lltemac_writeindirect $base $XTE_MIIMAI_OFFSET $MiiReg

    #poll
    set addr [format "0x%08x" [expr $base + $XTE_RDY_OFFSET]]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    set poll_count 0
    while {[expr $mval & $XTE_RSE_MIIM_RR_MASK] == 0} {
        set mval [format "0x%08x" [mem_read_word $addr] ]

        # do not poll forever, fail after 1000 tries.
        incr poll_count
        if {$poll_count == 1000} {
            puts "Timeout polling MII_Ready."
            return 0
        }
    } 

    # Read MII regval
    set retval [format "0x%04x" [mem_read_word [expr $base + $XTE_LSW_OFFSET]]]

    return $retval
}

# decode interrupt bits
proc lltemac_pint_intbits {regval} {
    # Read register offset definitions
    global env
    source "$env(XMD_USER_TCLDIR)/include/xlltemac_hw.tcl"

   if {$regval & $XTE_INT_HARDACSCMPLT_MASK} {
       puts -nonewline "HARDACSCMPLT "
   }
   if {$regval & $XTE_INT_AUTONEG_MASK} {
       puts -nonewline "AUTONEG "
   }
   if {$regval & $XTE_INT_RC_MASK} {
       puts -nonewline "RC "
   }
   if {$regval & $XTE_INT_RXRJECT_MASK} {
       puts -nonewline "RXRJECT "
   }
   if {$regval & $XTE_INT_RXFIFOOVR_MASK} {
       puts -nonewline "RXFIFOOVR "
   }
   if {$regval & $XTE_INT_TC_MASK} {
       puts -nonewline "TC "
   }
   if {$regval & $XTE_INT_RXDCM_LOCK_MASK} {
       puts -nonewline "RXDCM_LOCK "
   }
   puts ""
}

# print TEMAC registers
proc lltemac_print {base} {
    # Read register offset definitions
    global env
    source "$env(XMD_USER_TCLDIR)/include/xlltemac_hw.tcl"

    set addr [format "0x%08x" [expr $base + $XTE_RAF_OFFSET]]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    puts -nonewline "$addr RAF   $mval "
    if {$mval & $XTE_RAF_HTRST_MASK} {
        puts -nonewline "HTRST "
    }
    if {$mval & $XTE_RAF_MCSTREJ_MASK} {
        puts -nonewline "MCSTREJ "
    }
    if {$mval & $XTE_RAF_BCSTREJ_MASK} {
        puts -nonewline "BCSTREJ "
    }
    puts ""

    set addr [format "0x%08x" [expr $base + $XTE_TPF_OFFSET]]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    puts "$addr TPF   $mval"

    set addr [format "0x%08x" [expr $base + $XTE_IFGP_OFFSET]]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    puts "$addr IFGP  $mval"

    set addr [format "0x%08x" [expr $base + $XTE_IS_OFFSET]]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    puts -nonewline "$addr IS    $mval "
    lltemac_pint_intbits $mval

    set addr [format "0x%08x" [expr $base + $XTE_IP_OFFSET]]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    puts -nonewline "$addr IP    $mval "
    lltemac_pint_intbits $mval

    set addr [format "0x%08x" [expr $base + $XTE_IE_OFFSET]]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    puts -nonewline "$addr IE    $mval "
    lltemac_pint_intbits $mval

    set addr [format "0x%08x" [expr $base + $XTE_MSW_OFFSET]]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    puts "$addr MSW   $mval"

    set addr [format "0x%08x" [expr $base + $XTE_LSW_OFFSET]]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    puts "$addr LSW   $mval"

    set addr [format "0x%08x" [expr $base + $XTE_CTL_OFFSET]]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    puts "$addr CTL   $mval"

    set addr [format "0x%08x" [expr $base + $XTE_RDY_OFFSET]]
    set mval [format "0x%08x" [mem_read_word $addr] ]
    puts "$addr RDY   $mval"

    puts "TEMAC Indirect Registers"
    set mval [lltemac_readindirect $base $XTE_RCW0_OFFSET]
    puts "     $XTE_RCW0_OFFSET RCW0  $mval"
    set mval [lltemac_readindirect $base $XTE_RCW1_OFFSET]
    puts -nonewline "     $XTE_RCW1_OFFSET RCW1  $mval "
    if {$mval & $XTE_RCW1_JUM_MASK} {
        puts -nonewline "JUMBO "
    }
    if {$mval & $XTE_RCW1_FCS_MASK} {
        puts -nonewline "FCS "
    }
    if {$mval & $XTE_RCW1_RX_MASK} {
        puts -nonewline "RX_ENABLE "
    } else {
        puts -nonewline "RX_DISABLE "
    }
    if {$mval & $XTE_RCW1_VLAN_MASK} {
        puts -nonewline "VLAN_ENABLE "
    }
    if {$mval & $XTE_RCW1_HD_MASK} {
        puts -nonewline "HD "
    }
    if {$mval & $XTE_RCW1_LT_DIS_MASK} {
        puts -nonewline "LEN_TYPE "
    }
    set pause_addr [format "0x%04x" [expr $mval & 0xFFFF]]
    puts "PAUSE_ADDR:$pause_addr"

    set mval [lltemac_readindirect $base $XTE_TC_OFFSET]
    puts -nonewline "     $XTE_TC_OFFSET TC    $mval "
    if {$mval & $XTE_TC_JUM_MASK} {
        puts -nonewline "JUMBO "
    }
    if {$mval & $XTE_TC_FCS_MASK} {
        puts -nonewline "FCS "
    }
    if {$mval & $XTE_TC_TX_MASK} {
        puts -nonewline "TX_ENABLE "
    } else {
        puts -nonewline "TX_DISABLE "
    }
    if {$mval & $XTE_TC_VLAN_MASK} {
        puts -nonewline "VLAN_ENABLE "
    }
    if {$mval & $XTE_TC_HD_MASK} {
        puts -nonewline "HD "
    }
    if {$mval & $XTE_TC_IFG_MASK} {
        puts -nonewline "IFG "
    }
    puts ""

    set mval [lltemac_readindirect $base $XTE_FCC_OFFSET]
    puts -nonewline "     $XTE_FCC_OFFSET FCC   $mval "
    if {$mval & $XTE_FCC_FCRX_MASK} {
        puts -nonewline "FCRX "
    }
    if {$mval & $XTE_FCC_FCTX_MASK} {
        puts -nonewline "FCTX "
    }
    puts ""

    set mval [lltemac_readindirect $base $XTE_EMMC_OFFSET]
    puts -nonewline "     $XTE_EMMC_OFFSET EMMC  $mval "
    set linkspeed "10MB "
    if {$mval & 0x40000000} {
        set linkspeed "100MB "
    }
    if {$mval & 0x80000000} {
        set linkspeed "1000MB "
    }
    puts -nonewline $linkspeed
    if {$mval & $XTE_EMMC_RGMII_MASK} {
        puts -nonewline "RGMII "
    }
    if {$mval & $XTE_EMMC_SGMII_MASK} {
        puts -nonewline "SGMII "
    }
    if {$mval & $XTE_EMMC_GPCS_MASK} {
        # 1000BaseX mode enable
        puts -nonewline "GPCS "
    }
    if {$mval & $XTE_EMMC_HOST_MASK} {
        puts -nonewline "HOST "
    }
    puts ""

    set mval [lltemac_readindirect $base $XTE_PHYC_OFFSET]
    puts "     $XTE_PHYC_OFFSET PHYC  $mval"
    set mval [lltemac_readindirect $base $XTE_MC_OFFSET]
    puts -nonewline "     $XTE_MC_OFFSET MC    $mval "
    if {$mval & $XTE_MC_MDIOEN_MASK} {
        set clk_div [expr $mval - $XTE_MC_MDIOEN_MASK]
        puts -nonewline "MDIOEN CLK_DIV:$clk_div"
    }
    puts ""

    set mval [lltemac_readindirect $base $XTE_UAW0_OFFSET]
    puts "     $XTE_UAW0_OFFSET UAW0  $mval"
    set mval [lltemac_readindirect $base $XTE_UAW1_OFFSET]
    puts "     $XTE_UAW1_OFFSET UAW1  $mval"
    set mval [lltemac_readindirect $base $XTE_MAW0_OFFSET]
    puts "     $XTE_MAW0_OFFSET MAW0  $mval"
    set mval [lltemac_readindirect $base $XTE_MAW1_OFFSET]
    puts "     $XTE_MAW1_OFFSET MAW1  $mval"
    set mval [lltemac_readindirect $base $XTE_AFM_OFFSET]
    puts -nonewline "     $XTE_AFM_OFFSET AFM   $mval "
    if {$mval & $XTE_AFM_PM_MASK} {
        puts -nonewline "PROMISCUOUS "
    }
    puts ""

    set mval [lltemac_readindirect $base $XTE_TIS_OFFSET]
    puts "     $XTE_TIS_OFFSET TIS   $mval"
    set mval [lltemac_readindirect $base $XTE_TIE_OFFSET]
    puts "     $XTE_TIE_OFFSET TIE   $mval"
    set mval [lltemac_readindirect $base $XTE_MIIMWD_OFFSET]
    puts "     $XTE_MIIMWD_OFFSET MIIWD $mval"
    set mval [lltemac_readindirect $base $XTE_MIIMAI_OFFSET]
    puts "     $XTE_MIIMAI_OFFSET MIIAI $mval"
}

