###############################################################################
##
## Copyright (c) 2008 Xilinx, Inc.  All rights reserved.
##
## Xilinx, Inc.
## XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" AS A
## COURTESY TO YOU. BY PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
## ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION OR
## STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS IMPLEMENTATION
## IS FREE FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE RESPONSIBLE
## FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE FOR YOUR IMPLEMENTATION.
## XILINX EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH RESPECT TO
## THE ADEQUACY OF THE IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO
## ANY WARRANTIES OR REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
## FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY
## AND FITNESS FOR A PARTICULAR PURPOSE.
##
## Filename:    mb.tcl
## Author:      Brian Hill
##
## Description:
## Print Microblaze CPU registers, decoding config/status/version info
##
## $Id: mb.tcl,v 1.7 2008/06/19 14:38:32 bhill Exp $
##
###############################################################################

puts "mb_print"

proc mb_pvr11_print {} {
    set regval [string trimleft [srrd pvr11] "pvr11: "]
    set regval [format "0x%08x" 0x0$regval ]
    puts -nonewline "PVR11: $regval "

    set mb_mmu [expr [expr $regval & 0xC0000000] >> 30]
    switch $mb_mmu {
        0 {puts -nonewline "MMU_NONE "}
        1 {puts -nonewline "MMU_USER "}
        2 {puts -nonewline "MMU_PROTECT "}
        3 {puts -nonewline "MMU_VIRTUAL "}
    }

    set mb_tlbacc [expr [expr $regval & 0x00C00000] >> 22]
    switch $mb_mmu {
        0 {puts -nonewline "TLBACC_MINIMAL "}
        1 {puts -nonewline "TLBACC_READ "}
        2 {puts -nonewline "TLBACC_WRITE "}
        3 {puts -nonewline "TLBACC_FULL "}
    }
    
    puts ""
}

proc mb_pvr10_print {} {
    set regval [string trimleft [srrd pvr10] "pvr10: "]
    set regval [format "0x%08x" 0x0$regval ]
    puts -nonewline "PVR10: $regval "

    set mb_arch [expr [expr $regval & 0xFF000000] >> 24]
    puts -nonewline "ARCH_"
    switch $mb_arch {
        4   {puts "Virtex2"}
        5   {puts "Virtex2Pro"}
        6   {puts "Spartan3"}
        7   {puts "Virtex4"}
        8   {puts "Virtex5"}
        9   {puts "Spartan3E"}
        10  {puts "Spartan3A"}
        11  {puts "Spartan3AN"}
        12  {puts "Spartan3Adsp"}
        default {puts "$mb_arch"}
    }
}

proc mb_pvr5_print {} {
    set regval [string trimleft [srrd pvr5] "pvr5: "]
    set regval [format "0x%08x" 0x0$regval ]
    puts -nonewline "PVR5:  $regval "

    if {$regval & 0x80000000} {
        puts -nonewline "DCU "
    } else {
        puts -nonewline "NO_DCU "
    }
    if {$regval & 0x01000000} {
        puts -nonewline "DCW "
    }
    puts ""
}

proc mb_pvr4_print {} {
    set regval [string trimleft [srrd pvr4] "pvr4: "]
    set regval [format "0x%08x" 0x0$regval ]
    puts -nonewline "PVR4:  $regval "

    if {$regval & 0x80000000} {
        puts -nonewline "ICU "
    } else {
        puts -nonewline "NO_ICU "
    }
    if {$regval & 0x01000000} {
        puts -nonewline "ICW "
    }
    puts ""
}

proc mb_pvr2_print {} {
    set regval [string trimleft [srrd pvr2] "pvr2: "]
    set regval [format "0x%08x" 0x0$regval ]
    puts -nonewline "PVR2:  $regval "

    if {$regval & 0x80000000} {
        puts -nonewline "DOPB "
    }
    if {$regval & 0x40000000} {
        puts -nonewline "DLMB "
    }
    if {$regval & 0x20000000} {
        puts -nonewline "IOPB "
    }
    if {$regval & 0x10000000} {
        puts -nonewline "ILMB "
    }
    if {$regval & 0x08000000} {
        puts -nonewline "IRQEDGE "
    }
    if {$regval & 0x04000000} {
        puts -nonewline "IRQPOS "
    }
    if {$regval & 0x02000000} {
        puts -nonewline "DPLB "
    }
    if {$regval & 0x01000000} {
        puts -nonewline "IPLB "
    }
    if {$regval & 0x00800000} {
        puts -nonewline "INTERCON "
    }
    if {$regval & 0x00080000} {
        puts -nonewline "FSL "
    }
    if {$regval & 0x00040000} {
        puts -nonewline "FSLEXC "
    }
    if {$regval & 0x00020000} {
        puts -nonewline "MSR "
    }
    if {$regval & 0x00010000} {
        puts -nonewline "PCMP "
    }
    if {$regval & 0x00008000} {
        puts -nonewline "AREA "
    }
    if {$regval & 0x00004000} {
        puts -nonewline "BS "
    }
    if {$regval & 0x00002000} {
        puts -nonewline "DIV "
    }
    if {$regval & 0x00001000} {
        puts -nonewline "MUL "
    }
    if {$regval & 0x00000800} {
        puts -nonewline "FPU "
    }
    if {$regval & 0x00000400} {
        puts -nonewline "MUL64 "
    }
    if {$regval & 0x00000200} {
        puts -nonewline "FPU2 "
    }
    if {$regval & 0x00000100} {
        puts -nonewline "IPLBEXC "
    }
    if {$regval & 0x00000080} {
        puts -nonewline "DPLBEXC "
    }
    if {$regval & 0x00000040} {
        puts -nonewline "OP0EXC "
    }
    if {$regval & 0x00000020} {
        puts -nonewline "UNEXC "
    }
    if {$regval & 0x00000010} {
        puts -nonewline "OPEXC "
    }
    if {$regval & 0x00000008} {
        puts -nonewline "IOPBEXC "
    }
    if {$regval & 0x00000004} {
        puts -nonewline "DOPBEXC "
    }
    if {$regval & 0x00000002} {
        puts -nonewline "DIVEXC "
    }
    if {$regval & 0x00000001} {
        puts -nonewline "FPUEXC "
    }
    puts ""
}

proc mb_pvr_print {} {
    set regval [string trimleft [rrd msr] "msr: "]
    set regval [format "0x%08x" 0x0$regval ]
    if {[expr $regval & 0x00000400] == 0} {
        puts "No PVR support"
        return 0;
    }

    set regval [string trimleft [srrd pvr0] "pvr0: "]
    set regval [format "0x%08x" 0x0$regval ]
    puts -nonewline "PVR0:  $regval "

    if {$regval & 0x80000000} {
        puts -nonewline "PVR_FULL "
        set retval 1
    } else {
        puts -nonewline "PVR_BASIC "
        set retval 0
    }
    if {$regval & 0x40000000} {
        puts -nonewline "BS "
    }
    if {$regval & 0x20000000} {
        puts -nonewline "DIV "
    }
    if {$regval & 0x10000000} {
        puts -nonewline "MUL "
    }
    if {$regval & 0x08000000} {
        puts -nonewline "FPU "
    }
    if {$regval & 0x04000000} {
        puts -nonewline "EXC "
    }
    if {$regval & 0x02000000} {
        puts -nonewline "ICU "
    }
    if {$regval & 0x01000000} {
        puts -nonewline "DCU "
    }
    if {$regval & 0x00800000} {
        puts -nonewline "MMU "
    }
    set mb_ver [expr [expr $regval & 0xF00] >> 8]
    puts -nonewline "VER_"
    switch $mb_ver {
        1 {puts "5.00.a"}
        2 {puts "5.00.b"}
        3 {puts "5.00.c"}
        4 {puts "6.00.a"}
        5 {puts "7.00.a"}
        6 {puts "6.00.b"}
        7 {puts "7.00.b"}
        8 {puts "7.10.a"}
        9 {puts "7.10.b"}
        10 {puts "7.10.c"}
        default {puts "UNKNOWN_$mb_ver"}
    }

    return $retval
}

proc mb_esr_print {} {
    set regval [string trimleft [srrd esr] "esr: "]
    set regval [format "0x%08x" 0x0$regval ]
    puts -nonewline "ESR:   $regval EXCEPTION-"

    set ec_val [expr $regval & 0x1F]
    switch $ec_val {
        0  {puts "FSL "}
        1  {puts "UNALIGNED_DATA "}
        2  {puts "ILLEGAL_OPCODE "}
        3  {puts "INSTR_BUS_ERR "}
        4  {puts "DATA_BUS_ERR "}
        5  {puts "DIV_0 "}
        6  {puts "FLOAT_PT "}
        7  {puts "PRIV_INSTR "}
        16 {puts "DATA_STORE "}
        17 {puts "INSTR_STORE "}
        18 {puts "DATA_TLB_MISS "}
        19 {puts "INSTR_TLB_MISS "}
        default {puts "UNKNOWN"}
    }
}

proc mb_msr_print {} {
    set regval [string trimleft [rrd msr] "msr: "]
    set regval [format "0x%08x" 0x0$regval ]
    puts -nonewline "MSR:   $regval "
    if {[expr $regval & 0x00000400] == 0} {
        # No PVR support.  Yuck.
        set retval 0
    } else {
        set retval 1
    }

    if {$regval & 0x00000001} {
        puts -nonewline "BE "
    }
    if {$regval & 0x00000002} {
        puts -nonewline "IE "
    }
    if {$regval & 0x00000004} {
        puts -nonewline "C "
    }
    if {$regval & 0x00000008} {
        puts -nonewline "BIP "
    }
    if {$regval & 0x00000010} {
        puts -nonewline "FSL "
    }
    if {$regval & 0x00000020} {
        puts -nonewline "ICE "
    }
    if {$regval & 0x00000040} {
        puts -nonewline "DZ "
    }
    if {$regval & 0x00000080} {
        puts -nonewline "DCE "
    }
    if {$regval & 0x00000100} {
        puts -nonewline "EE "
    }
    if {$regval & 0x00000200} {
        puts -nonewline "EIP "
        set eip 1
    } else {
        set eip 0
    }
    if {$regval & 0x00000400} {
        puts -nonewline "PVR "
    } else {
        puts -nonewline "NO_PVR "
    }
    if {$regval & 0x00000800} {
        puts -nonewline "UM "
    }
    if {$regval & 0x00001000} {
        puts -nonewline "UMS "
    }
    if {$regval & 0x00002000} {
        puts -nonewline "VM "
    }
    if {$regval & 0x00004000} {
        puts -nonewline "VMS "
    }
    puts ""

    if {$eip} {
        mb_esr_print
    }

    return $retval
}

proc mb_print {} {
    set pvr_basic [mb_msr_print]
    if {$pvr_basic} {
        set pvr_full [mb_pvr_print]
        if {$pvr_full == 1} {
            mb_pvr2_print
            mb_pvr4_print
            mb_pvr5_print
            mb_pvr10_print
            mb_pvr11_print
        }
    } else {
        puts "No PVR support."
    }
}
