###############################################################################
##
## Copyright (c) 2008 Xilinx, Inc.  All rights reserved.
##
## Xilinx, Inc.
## XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" AS A
## COURTESY TO YOU. BY PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
## ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION OR
## STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS IMPLEMENTATION
## IS FREE FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE RESPONSIBLE
## FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE FOR YOUR IMPLEMENTATION.
## XILINX EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH RESPECT TO
## THE ADEQUACY OF THE IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO
## ANY WARRANTIES OR REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
## FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY
## AND FITNESS FOR A PARTICULAR PURPOSE.
##
## Filename:    mb_dis.tcl
## Author:      Brian Hill
##
## Description:
## Prints a disassembly of instructions at the specified address.
## The mb-objcopy and mb-objdump executables are used to produce the
## disassembly -- the must be in the PATH for this script to operate properly.
##
## $Id: $
##
###############################################################################
#
# EXAMPLE USAGE:
# 
# XMD% mb_dis 0x1a8 6
#  1a8:   30a010f0        addik   r5, r0, 4336
#  1ac:   3021ffe4        addik   r1, r1, -28
#  1b0:   f9e10000        swi     r15, r1, 0
#  1b4:   b9f400dc        brlid   r15, 220
#  1b8:   80000000        or      r0, r0, r0
#  1bc:   30a01108        addik   r5, r0, 4360
# 
# As compared to:
# mb-objdump -d foo.elf
#
# 000001a8 <main>:
#   1a8:       30a010f0        addik   r5, r0, 4336    // 10f0 <__rodata_start>
#   1ac:       3021ffe4        addik   r1, r1, -28
#   1b0:       f9e10000        swi     r15, r1, 0
#   1b4:       b9f400dc        brlid   r15, 220        // 290 <print>
#   1b8:       80000000        or      r0, r0, r0
#   1bc:       30a01108        addik   r5, r0, 4360
#

puts "mb_dis"

# hex_to_bin:
# Convert HEX string to binary data
proc hex_to_bin {hex} {
  set binval [binary format "H*" $hex]
  return $binval
}

# memsave:
# save $words of memoray at $addr to binary $file_out
proc memsave {addr words file_out} {

    set f [open $file_out w]
    fconfigure $f \
        -translation binary \
        -encoding binary

    set word 0
    while {$word < $words} {
        set memval [format "%08x" [mem_read_word $addr]]
        set binval [hex_to_bin $memval]
        puts -nonewline $f $binval

        incr addr 4
        incr word
    }

    close $f
}

# mb_dis:
# Disassemble $words of MicroBlaze instructions starting at $startaddr
proc mb_dis {startaddr words} {

    # Save specified memory to a binary file
    memsave $startaddr $words "memory.bin"

    # Convert binary file to MicroBlaze object file
    exec mb-objcopy -I binary -O elf32-microblaze -B microblaze \
         --rename-section .data=.text \
         --change-section-address .data=$startaddr memory.bin memory.o

    # Disassemble MicroBlaze object file
    set f1 [open "|mb-objdump -D memory.o" r]

    # Skip initial crud
    while {[eof $f1] == 0} {
        gets $f1 line
        if {[regexp {_binary_memory_bin_start} $line text1 ]} {
            break
        }
    }

    # Display disassembly
    while {[eof $f1] == 0} {
        gets $f1 line
        puts "$line"
    }

    close $f1
    
    # Clean up temporary files
    # The TCL "file delete" procedure does not seem to operate as expected 
    exec rm memory.o
    exec rm memory.bin
}
