###############################################################################
##
## Copyright (c) 2007, 2008 Xilinx, Inc.  All rights reserved.
##
## Xilinx, Inc.
## XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" AS A
## COURTESY TO YOU. BY PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
## ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION OR
## STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS IMPLEMENTATION
## IS FREE FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE RESPONSIBLE
## FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE FOR YOUR IMPLEMENTATION.
## XILINX EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH RESPECT TO
## THE ADEQUACY OF THE IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO
## ANY WARRANTIES OR REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
## FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY
## AND FITNESS FOR A PARTICULAR PURPOSE.
##
## Filename:    ppc405.tcl
## Author:      Brian Hill
##
## Description:
## Print PPC405 config registers, decoding some interesting bits.
##
## $Id: ppc405.tcl,v 1.3 2008/03/11 17:09:09 bhill Exp $
##
###############################################################################

puts "ppc405_print"
puts "ppc405_rm_sa_print"

# Read PPC405 MSR Register and examine the EE bit.  
# Print in plain English if External Exceptions(interrupts) are presently
# enabled.
proc ppc405_intenable_print {} {
    set regval [string trimleft [rrd msr] "msr: "]
    set regval [format "0x%08x" 0x$regval ]

    puts -nonewline "PPC External Interrupts "
    if {$regval & 0x00008000} {
        puts "ENABLED"
    } else {
        puts "DISABLED"
    }
}

# print and decode contents of Processor Version Register
proc ppc405_pvr_print {} {
    set regval [string trimleft [srrd pvr] "pvr: "]
    set regval [format "0x%08x" 0x$regval ]
    puts -nonewline "PVR:  $regval "
 
    set own [expr $regval >> 24]
    if {$own != 0x20} {
        puts "NON-XILINX "
        return
    }

    set pcf [expr [expr $regval >> 16] & 0xF]
    if {$pcf == 1} {
        puts -nonewline "PPC405 "
    } else {
        puts "Unknown Core Family"
        return
    }

    set cas [expr [expr $regval >> 10] & 0x3F]
    if {$cas == 2} {
        puts -nonewline "16K cache "
    } else {
        puts -nonewline "CacheArraySize_$cas "
    }

    set pcl [expr [expr $regval >> 6] & 0xF]
    puts -nonewline "Revision $pcl "

    set aid [expr [expr $regval >> 4] & 0x3]
    switch $aid {
        0 {puts -nonewline "UNKNOWN_AID "}
        1 {puts -nonewline "UNKNOWN_AID "}
        2 {puts -nonewline "Virtex-II Pro "}
        3 {puts -nonewline "Virtex-4 "}
    }

    puts ""
}

# Print and decode contents of Machine State Register
proc ppc405_msr_print {} {
    set regval [string trimleft [rrd msr] "msr: "]
    set regval [format "0x%08x" 0x$regval ]
    puts -nonewline "MSR:  $regval "

    # WE Wait State Enable
    if {$regval & 0x00040000} {
        puts -nonewline "WE "
    }
    # CE Critical Interrupt Enable
    if {$regval & 0x00020000} {
        puts -nonewline "CE "
    }
    # EE External Interrupt Enable
    if {$regval & 0x00008000} {
        puts -nonewline "EE "
    }
    # PR Privilege Mode
    if {$regval & 0x00004000} {
        puts -nonewline "PRuser "
    } else {
        puts -nonewline "PRpriv "
    }
    # ME Machine Check Enable
    if {$regval & 0x00001000} {
        puts -nonewline "ME "
    }
    # DWE Debug Wait Enable
    if {$regval & 0x00000400} {
        puts -nonewline "DWE "
    }
    # DE Debug Interrupt Enable
    if {$regval & 0x00000200} {
        puts -nonewline "DE "
    }

    # IR Instruction Relocate
    if {$regval & 0x00000020} {
        puts -nonewline "IR "
    }
    # DR Data Relocate
    if {$regval & 0x00000010} {
        puts -nonewline "DR "
    }
    if  { [expr $regval & 0x00000020] ||
          [expr $regval & 0x00000010] } {
        puts -nonewline "(Address Translation) "
        set mode "virtual"
    } else {
        puts -nonewline "(Real Mode) "
        set mode "real"
    }

    puts ""

    return $mode
}

# Print and decode contents of TCR Timer Control Register
proc ppc405_tcr_print {} {
    set regval [string trimleft [srrd tcr] "tcr: "]
    set regval [format "0x%08x" 0x$regval ]
    puts -nonewline "TCR:  $regval "

    # WIE Watchdog Interrupt Enable
    if {$regval & 0x08000000} {
        puts -nonewline "WIE "
    }
    # PIE PIT-Interrupt Enable
    if {$regval & 0x04000000} {
        puts -nonewline "PIE "
    }

    puts ""
}

# Print and decode contents of TSR Timer Status Register
proc ppc405_tsr_print {} {
    set regval [string trimleft [srrd tsr] "tsr: "]
    set regval [format "0x%08x" 0x$regval ]
    puts -nonewline "TSR:  $regval "

    # ENW Enable Next Watchdog
    if {$regval & 0x80000000} {
        puts -nonewline "ENW "
    }
    # WIS Watchdog Interrupt Status
    if {$regval & 0x04000000} {
        puts -nonewline "WIS "
    }

    puts ""
}

# Print and decode contents of CCR0 Core Configuration Register 0
proc ppc405_ccr0_print {} {
    set regval [string trimleft [srrd ccr0] "ccr0: "]
    set regval [format "0x%08x" 0x$regval ]
    puts -nonewline "CCR0: $regval "

    # LWL Load Word as Line
    if {$regval & 0x02000000} {
        puts -nonewline "LWL "
    }
    # LWOA Load Without Allocate
    if {$regval & 0x01000000} {
        puts -nonewline "LWOA "
    }
    # SWOA Store Without Allocate
    if {$regval & 0x00800000} {
        puts -nonewline "SWOA "
    }
    # DPP1 DCU PLB-Priority Bit 1
    if {$regval & 0x00400000} {
        puts -nonewline "DPP1 "
    }
    # DPE Data Cache Priority Enable
    if {$regval & 0x00080000} {
        puts -nonewline "DPE "
    }
    # DPP Data Cache Priority Precision
    if {$regval & 0x00040000} {
        puts -nonewline "DPP "
    }
    # U0XE Enable U0 Exception
    if {$regval & 0x00020000} {
        puts -nonewline "U0XE "
    }
    # LBDE Load Debug Enable
    if {$regval & 0x00010000} {
        puts -nonewline "LBDE "
    }
    # IPE Instruction Cache Parity Enable
    if {$regval & 0x00002000} {
        puts -nonewline "IPE "
    }
    # TPE TLB Parity Enable
    if {$regval & 0x00001000} {
        puts -nonewline "TPE "
    }
    # PFC Prefetching for Cacheable Regions
    if {$regval & 0x00000800} {
        puts -nonewline "PFC "
    }
    # PFNC Prefetching for Non-Cacheable Regions
    if {$regval & 0x00000400} {
        puts -nonewline "PFNC "
    }
    # NCRS Non-Cacheable Request Size
    if {$regval & 0x00000200} {
        puts -nonewline "NCRS "
    }
    # FWOA Fetch Without Allocate
    if {$regval & 0x00000100} {
        puts -nonewline "FWOA "
    }
    # CIS Cache Information Select
    if {$regval & 0x00000010} {
        puts -nonewline "CIS "
    }
    # PRS Parity Select
    if {$regval & 0x00000008} {
        puts -nonewline "PRS "
    }
    # CWS Cache-Way Select
    if {$regval & 0x00000001} {
        puts -nonewline "CWS "
    }
 
    puts ""
}

# Exception Syndrome Register
proc ppc405_esr_print {} {
    set regval [string trimleft [srrd esr] "esr: "]
    set regval [format "0x%08x" 0x$regval ]
    puts -nonewline "ESR:  $regval "

    if {$regval & 0x80000000} {
        # Machine Check - Instruction
        puts -nonewline "MCI "
    }
    if {$regval & 0x08000000} {
        # Program - Illegal Instruction
        puts -nonewline "PIL "
    }
    if {$regval & 0x04000000} {
        # Program - Privileged Instruction
        puts -nonewline "PPR "
    }
    if {$regval & 0x02000000} {
        # Program - Trap Instruction
        puts -nonewline "PTR "
    }
    if {$regval & 0x01000000} {
        # Program - Unimplemented Instruction
        puts -nonewline "PEU "
    }
    if {$regval & 0x00800000} {
        # Data Storage - Store Instruction
        puts -nonewline "DST "
    }
    if {$regval & 0x00400000} {
        # Data and Instruction Storage - Zone Protection
        puts -nonewline "DIZ "
    }
    if {$regval & 0x00080000} {
        # Program - Floating Point Insruction
        puts -nonewline "PFP "
    }
    if {$regval & 0x00040000} {
        # Program - Auxiliary Processor Instruction
        puts -nonewline "PAP "
    }
    if {$regval & 0x00008000} {
        # Data Storage - U0 Protection
        puts -nonewline "U0F "
    }
    puts ""
}

# Display real mode memory storage attribute registers
proc ppc405_rm_sa_reg_print {} {
    set regval [string trimleft [srrd dcwr] "dcwr: "]
    set regval [format "0x%08x" 0x0$regval ]
    puts "DCWR: $regval "

    set regval [string trimleft [srrd dccr] "dccr: "]
    set regval [format "0x%08x" 0x0$regval ]
    puts "DCCR: $regval "

    set regval [string trimleft [srrd iccr] "iccr: "]
    set regval [format "0x%08x" 0x0$regval ]
    puts "ICCR: $regval "

    set regval [string trimleft [srrd sgr] "sgr: "]
    set regval [format "0x%08x" 0x0$regval ]
    puts "SGR:  $regval "

    set regval [string trimleft [srrd su0r] "su0r: "]
    set regval [format "0x%08x" 0x0$regval ]
    puts "SU0R: $regval "

    set regval [string trimleft [srrd sler] "sler: "]
    set regval [format "0x%08x" 0x0$regval ]
    puts "SLER: $regval "
}

# Display real mode memory storage attributes
proc ppc405_rm_sa_print {} {
    # Data Cache Write-Through Register
    set regval [string trimleft [srrd dcwr] "dcwr: "]
    set dcwr [format "0x%08x" 0x0$regval ]

    # Data Cache Cacheability Register
    set regval [string trimleft [srrd dccr] "dccr: "]
    set dccr [format "0x%08x" 0x0$regval ]

    # Instruction Cache Cacheability Register
    set regval [string trimleft [srrd iccr] "iccr: "]
    set iccr [format "0x%08x" 0x0$regval ]

    # Storage Guarded Register
    set regval [string trimleft [srrd sgr] "sgr: "]
    set sgr [format "0x%08x" 0x0$regval ]

    # Storage User-Defined 0 Register
    set regval [string trimleft [srrd su0r] "su0r: "]
    set su0r [format "0x%08x" 0x0$regval ]

    # Storage Little-Endian Register
    set regval [string trimleft [srrd sler] "sler: "]
    set sler [format "0x%08x" 0x0$regval ]

    puts -nonewline "Address    D-Cacheable Write-Through I-Cacheable Guarded "
    puts "U0 Little-Endian"

    set address       0x00000000
    set address_range 0x08000000
    set bit 0

    while { $bit < 32 } {
        puts -nonewline "$address "

        if { [expr 1 << [expr 31 - $bit]] & $dccr } {
            puts -nonewline "          X "
        } else {
            puts -nonewline "            "
        }

        if { [expr 1 << [expr 31 - $bit]] & $dcwr } {
            puts -nonewline "            X "
        } else {
            puts -nonewline "              "
        }

        if { [expr 1 << [expr 31 - $bit]] & $iccr } {
            puts -nonewline "          X "
        } else {
            puts -nonewline "            "
        }

        if { [expr 1 << [expr 31 - $bit]] & $sgr } {
            puts -nonewline "      X "
        } else {
            puts -nonewline "        "
        }

        if { [expr 1 << [expr 31 - $bit]] & $su0r } {
            puts -nonewline " X "
        } else {
            puts -nonewline "   "
        }

        if { [expr 1 << [expr 31 - $bit]] & $sler } {
            puts -nonewline "            X "
        } else {
            puts -nonewline "              "
        }

        puts ""
        incr bit
        set address [format "0x%08x" [expr $address + $address_range]]
    }
}

proc ppc405_print {} {
    ppc405_pvr_print
    set mode [ppc405_msr_print]
    ppc405_tcr_print
    ppc405_tsr_print
    ppc405_ccr0_print
    ppc405_esr_print

    if {$mode == "real"} {
        puts "Real mode storage attribute registers:"
        ppc405_rm_sa_reg_print
    }
}
