#!/usr/bin/tclsh
# Interpret script with TCLSH, or, make below the 1st line run script with XMD
#!xmd -tcl

###############################################################################
##
## Copyright (c) 2007 Xilinx, Inc.  All rights reserved.
##
## Xilinx, Inc.
## XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" AS A
## COURTESY TO YOU. BY PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
## ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION OR
## STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS IMPLEMENTATION
## IS FREE FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE RESPONSIBLE
## FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE FOR YOUR IMPLEMENTATION.
## XILINX EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH RESPECT TO
## THE ADEQUACY OF THE IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO
## ANY WARRANTIES OR REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
## FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY
## AND FITNESS FOR A PARTICULAR PURPOSE.
##
## Filename: stackscan.tcl
## Author:   Brian Hill
##
## Description:
## This script parses the text from a crash (produce by xxx_crashtrace.c)
## It will look up addresses in the symbol table and print them out, with offset
## (if any).
##
## $Id: stackscan.tcl,v 1.2 2008/04/21 20:48:25 bhill Exp $
##
###############################################################################

## EXAMPLE:
## stackscan.tcl TestApp_StackOverflow.crash TestApp_StackOverflow.sym
## EXCEPTION ADDR: 0x00006E38 B crash_instructions
## FRAME:          0x00002650 t ProgramExceptionHandler+324
## FRAME:          0x00000778 T _vector0700+120
## FRAME:          0x000022B4 T crash_function+32
## FRAME:          0x00002294 T crash_function
## FRAME:          0x000023EC T myfunction1+36
## FRAME:          0x0000245C T main+76
## FRAME:          0x00002188 T __vectors_end+196
## 


if {$argc != 2} {
    error "Usage: $argv0 <crashoutput> <symbolfile>"
}

set crashfile [lindex $argv 0]
set symfile [lindex $argv 1]
set fcrash [open $crashfile r]

#
# Search through the symbol file for a match to the supplied address
proc lookup_sym {addr} {
 
    global symfile
    set fsym [open $symfile r]

    set prevsym ""
    set prevsymaddr 0x0
    set prevsymtype "T"

    set addr    [string tolower $addr]

    # read each line of the symbol fil
    while {[gets $fsym line] >= 0} {
        # only look at lines with symbols + addresses
        if {[regexp {([0-f]+) ([a-zA-Z]) (.*)} $line \
            text1 symaddr symtype symbol]} {

            set symaddr [format "0x%08x" 0x$symaddr]
            set symaddr [string tolower $symaddr]

            # exact match ?
            if {$symaddr == $addr} {
                close $fsym
                return "$symtype $symbol"
            }

            # Compare the symbol address to the return address as simple
            # strings rather than arithmetically ($symaddr > $addr).
          # if {$symaddr > $addr} 
            if { [string compare $symaddr $addr] == 1} {
                close $fsym
                return  "$prevsymtype $prevsym+[expr $addr - $prevsymaddr]"
            }

            # Ignore Absolute addresses, as they may be just constants and
            # not really addresses at all
            if {$symtype != "A"} {
                # remember previous symbol for inexact matches (foo()+offset)
                set prevsym $symbol
                set prevsymaddr $symaddr
                set prevsymtype $symtype
            }
        }
    }
    
    close $fsym
}

# 
# Examine each line of the crash output.  Parse addresses and look them up
# in the symbol table file with the lookup_sym procedure
while {[gets $fcrash line] >= 0} {
    if {[regexp {Exception at: (0x[0-f]+)} $line text1 addr]} {
        puts "EXCEPTION ADDR: $addr [lookup_sym $addr]"
    }
    if {[regexp {^FRAME: (0x[0-f]+).RETURN.(0x[0-f]+)} $line \
            text1 frame return]} {
        puts "FRAME:          $return [lookup_sym $return]"
    }
}


