###############################################################################
##
## Copyright (c) 2007 Xilinx, Inc.  All rights reserved.
##
## Xilinx, Inc.
## XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" AS A
## COURTESY TO YOU. BY PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
## ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION OR
## STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS IMPLEMENTATION
## IS FREE FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE RESPONSIBLE
## FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE FOR YOUR IMPLEMENTATION.
## XILINX EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH RESPECT TO
## THE ADEQUACY OF THE IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO
## ANY WARRANTIES OR REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
## FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY
## AND FITNESS FOR A PARTICULAR PURPOSE.
##
## Filename:    uartns550.tcl
##
## Description:
## Print Uart550 controller registers
##
## $Id: uartns550.tcl,v 1.1 2007/11/14 21:01:59 bhill Exp $
##
###############################################################################


puts "uartns550_print"

# Print UART NS550 information
proc uartns550_print {base} {
    # Read register offset definitions
    global env
    source "$env(XMD_USER_TCLDIR)/include/xuartns550_l.tcl"

    set addr [format "0x%08x" [expr $base + $XUN_RBR_OFFSET]]
    set mval [format "0x%02x" [mem_read_word $addr] ]
    puts "$addr RBR   $mval"

    set addr [format "0x%08x" [expr $base + $XUN_IER_OFFSET]]
    set mval [format "0x%02x" [mem_read_word $addr] ]
    puts -nonewline "$addr IER   $mval "
    if {$mval & $XUN_IER_MODEM_STATUS} {
        puts -nonewline "MODEM_STATUS "
    }
    if {$mval & $XUN_IER_RX_LINE} {
        puts -nonewline "RX_LINE "
    }
    if {$mval & $XUN_IER_TX_EMPTY} {
        puts -nonewline "TX_EMPTY "
    }
    if {$mval & $XUN_IER_RX_DATA} {
        puts -nonewline "RX_DATA "
    }
    puts ""

    set addr [format "0x%08x" [expr $base + $XUN_IIR_OFFSET]]
    set mval [format "0x%02x" [mem_read_word $addr] ]
    puts -nonewline "$addr IIR   $mval "
    if {$mval & 0xC0} {
        puts -nonewline "FIFOEN "
    }
    set iir_intid [expr [expr $mval & 0x0E] >> 1]
    switch $iir_intid {
        0 {puts -nonewline "MODEM_STATUS "}
        1 {puts -nonewline "TX_EMPTY "}
        2 {puts -nonewline "RX_DATA "}
        3 {puts -nonewline "RX_LINE "}
        4 {puts -nonewline "INVALID "}
        5 {puts -nonewline "INVALID "}
        6 {puts -nonewline "CHAR_TIMEOUT "}
    }
    if {$mval & 0x01} {
        puts -nonewline "INTPEND "
    }
    puts ""

    set addr [format "0x%08x" [expr $base + $XUN_LCR_OFFSET]]
    set mval [format "0x%02x" [mem_read_word $addr] ]
    puts -nonewline "$addr LCR   $mval "
    if {$mval & $XUN_LCR_DLAB} {
        puts -nonewline "DLAB "
    }
    if {$mval & $XUN_LCR_SET_BREAK} {
        puts -nonewline "SET_BREAK "
    }
    if {$mval & $XUN_LCR_STICK_PARITY} {
        puts -nonewline "STICK_PARITY "
    }
    if {$mval & $XUN_LCR_EVEN_PARITY} {
        puts -nonewline "EVEN_PARITY "
    }
    if {$mval & $XUN_LCR_ENABLE_PARITY} {
        puts -nonewline "ENABLE_PARITY "
    } else {
        puts -nonewline "NO_PARITY "
    }
    if {$mval & $XUN_LCR_2_STOP_BITS} {
        puts -nonewline "2_STOP_BITS "
    } else {
        puts -nonewline "1_STOP_BIT "
    }
    set lcr_wordlen [expr $mval & $XUN_LCR_LENGTH_MASK] 
    switch $lcr_wordlen {
        0 {puts -nonewline "5bits "}
        1 {puts -nonewline "6bits "}
        2 {puts -nonewline "7bits "}
        3 {puts -nonewline "8bits "}
    }
    puts ""

    set addr [format "0x%08x" [expr $base + $XUN_MCR_OFFSET]]
    set mval [format "0x%02x" [mem_read_word $addr] ]
    puts -nonewline "$addr MCR   $mval "
    if {$mval & $XUN_MCR_LOOP} {
        puts -nonewline "LOOP "
    }
    if {$mval & $XUN_MCR_RTS} {
        puts -nonewline "RTS "
    }
    if {$mval & $XUN_MCR_DTR} {
        puts -nonewline "DTR "
    }
    puts ""

    set addr [format "0x%08x" [expr $base + $XUN_LSR_OFFSET]]
    set mval [format "0x%02x" [mem_read_word $addr] ]
    puts -nonewline "$addr LSR   $mval "
    if {$mval & $XUN_LSR_RX_FIFO_ERROR} {
        puts -nonewline "RX_FIFO_ERROR "
    }
    if {$mval & $XUN_LSR_TX_EMPTY} {
        puts -nonewline "TX_EMPTY "
    }
    if {$mval & $XUN_LSR_TX_BUFFER_EMPTY} {
        puts -nonewline "TX_BUFFER_EMPTY "
    }
    if {$mval & $XUN_LSR_BREAK_INT} {
        puts -nonewline "BREAK_INT "
    }
    if {$mval & $XUN_LSR_FRAMING_ERROR} {
        puts -nonewline "FRAMING_ERROR "
    }
    if {$mval & $XUN_LSR_PARITY_ERROR} {
        puts -nonewline "PARITY_ERROR "
    }
    if {$mval & $XUN_LSR_OVERRUN_ERROR} {
        puts -nonewline "OVERRUN_ERROR "
    }
    if {$mval & $XUN_LSR_DATA_READY} {
        puts -nonewline "DATA_READY "
    }
    puts ""

    set addr [format "0x%08x" [expr $base + $XUN_MSR_OFFSET]]
    set mval [format "0x%02x" [mem_read_word $addr] ]
    puts -nonewline "$addr MSR   $mval "
    if {$mval & 0x80} {
        puts -nonewline "DCD "
    }
    if {$mval & 0x40} {
        puts -nonewline "RI "
    }
    if {$mval & 0x20} {
        puts -nonewline "DSR "
    }
    if {$mval & 0x10} {
        puts -nonewline "CTS "
    }
    if {$mval & 0x08} {
        puts -nonewline "DDCD "
    }
    if {$mval & 0x04} {
        puts -nonewline "TERI "
    }
    if {$mval & 0x02} {
        puts -nonewline "DDSR "
    }
    if {$mval & 0x01} {
        puts -nonewline "DCTS "
    }
    puts ""

    # Raise DLAB
    set addr [format "0x%08x" [expr $base + $XUN_LCR_OFFSET]]
    set lcr_reg [format "0x%02x" [mem_read_word $addr] ]
    if {[expr $lcr_reg & $XUN_LCR_DLAB] == 0} {
        mwr $addr [expr $lcr_reg + $XUN_LCR_DLAB]
    }

    set addr [format "0x%08x" [expr $base + $XUN_DLM_OFFSET]]
    set dlm_val [format "0x%02x" [mem_read_word $addr] ]
    puts "$addr DLM   $dlm_val "
    set addr [format "0x%08x" [expr $base + $XUN_DLL_OFFSET]]
    set dll_val [format "0x%02x" [mem_read_word $addr] ]
    puts -nonewline "$addr DLL   $dll_val "
    set divisor [expr [expr $dlm_val << 8] + $dll_val]
    set divisor [format "0x%02x" $divisor]
    puts "DIVISOR: $divisor (BPS = FREQ/(DIVISOR << 4))"
    
    # lower DLAB
    set addr [format "0x%08x" [expr $base + $XUN_LCR_OFFSET]]
    mwr $addr $lcr_reg 
}
