
puts "mem_read_byte"
puts "mem_write_byte"
puts "mem_read_hwd"
puts "memcpy"

# mem_read_byte
# return a single byte from the specified address
proc mem_read_byte {address} {
    set wordaddr [expr $address & 0xfffffffc]
    set data [format "0x%08x" [mem_read_word $wordaddr]]

    set whichbyte [expr $address - $wordaddr]
    switch $whichbyte {
        0 { set byteval [expr [expr $data >> 24] & 0xff] }
        1 { set byteval [expr [expr $data >> 16] & 0xff] }
        2 { set byteval [expr [expr $data >>  8] & 0xff] }
        3 { set byteval [expr [expr $data >>  0] & 0xff] }
    }

    return $byteval
}

# mem_write_byte
# modify a single byte at the specified address 
proc mem_write_byte {address data} {
    set wordaddr [expr $address & 0xfffffffc]
    set readdata [format "0x%08x" [mem_read_word $wordaddr]]

    set whichbyte [expr $address - $wordaddr]
    switch $whichbyte {
        0 { set byteval [expr $data << 24]
            set readdata [expr $readdata & 0x00FFFFFF]
          }
        1 { set byteval [expr $data << 16]
            set readdata [expr $readdata & 0xFF00FFFF]
          }
        2 { set byteval [expr $data << 8]
            set readdata [expr $readdata & 0xFFFF00FF]
          }
        3 { set byteval [expr $data << 0]
            set readdata [expr $readdata & 0xFFFFFF00]
          }
    }

    set writedata [format "0x%08x" [expr $readdata + $byteval]]
    mem_write_word $wordaddr $writedata
}

# memcpy
# copy <bytes> from <sourceaddr> to <destaddr> with any alignment
proc memcpy {sourceaddr destaddr bytes} {
    while {$bytes > 0} {
        # read from source
        set byte [mem_read_byte $sourceaddr]
        # write to destination
        mem_write_byte $destaddr $byte

        # next byte
        incr sourceaddr 1
        incr destaddr 1
        set bytes [expr $bytes - 1]
    }
}

# read a half-word (16 bits)
proc mem_read_hwd {bufaddr} {
    # should we concern ourselves with half-words?
    set is_half_word [expr $bufaddr & 0x2]   

    set bufaddr [expr $bufaddr & 0xfffffffc]
    set val [mem_read_word $bufaddr]

    if {$is_half_word} {
        set val [expr $val & 0xFFFF]
    } else {
        set val [expr $val >> 16]
        set val [expr $val & 0xFFFF]
    }
    
    set val [format "0x%04x" $val]
    return $val
}

# read two half-words and return a whole word
proc rdwd_by_hw {bufaddr} {
    
    set val_msw [expr [mem_read_hwd $bufaddr] << 16]
    incr bufaddr 2

    set val_lsw [mem_read_hwd $bufaddr]
    set val [expr $val_msw + $val_lsw]

    set val [format "0x%08x" $val]
    return $val
}

